@@ -9,13 +9,17 @@
 ######################################################################
 
 import argparse
+import locale
 import re
+import sys
 import textwrap
 
 import arrow
 from rst2ansi import rst2ansi
 from b2sdk.v1 import RetentionPeriod
 
+_arrow_version = tuple(int(p) for p in arrow.__version__.split("."))
+
 
 class RawTextHelpFormatter(argparse.RawTextHelpFormatter):
     """
@@ -68,7 +72,10 @@ class ArgumentParser(argparse.ArgumentParser):
             if self._for_docs:
                 self._description = textwrap.dedent(self._raw_description)
             else:
-                self._description = rst2ansi(self._raw_description.encode('utf-8'))
+                encoding = self._get_encoding()
+                self._description = rst2ansi(
+                    self._raw_description.encode(encoding), output_encoding=encoding
+                )
 
         return self._description
 
@@ -78,22 +85,36 @@ class ArgumentParser(argparse.ArgumentParser):
 
     def error(self, message):
         self.print_help()
+
         args = {'prog': self.prog, 'message': message}
         self.exit(2, '\n%(prog)s: error: %(message)s\n' % args)
 
+    @classmethod
+    def _get_encoding(cls):
+        _, locale_encoding = locale.getdefaultlocale()
+        if locale_encoding is not None:
+            return sys.getdefaultencoding()
+
+        # locales are improperly configured
+        return 'ascii'
+
 
 def parse_comma_separated_list(s):
     """
     Parse comma-separated list.
     """
-    return [word.strip() for word in s.split(',')]
+    return [word.strip() for word in s.split(",")]
 
 
 def parse_millis_from_float_timestamp(s):
     """
     Parse timestamp, e.g. 1367900664 or 1367900664.152
     """
-    return int(arrow.get(float(s)).format('XSSS'))
+    parsed = arrow.get(float(s))
+    if _arrow_version < (1, 0, 0):
+        return int(parsed.format("XSSS"))
+    else:
+        return int(parsed.format("x")[:13])
 
 
 def parse_range(s):
